@@ -1,7 +1,4 @@
-import copy
-
-
-class JciHitachiStatus:
+class JciHitachiStatus:  # pragma: no cover
     idx = {}
 
     def __init__(self, status, default) -> None:
@@ -21,7 +18,7 @@ class JciHitachiStatus:
         return dict((key, getattr(self, key)) for key in self.idx)
 
 
-class JciHitachiAC(JciHitachiStatus):
+class JciHitachiAC(JciHitachiStatus):  # pragma: no cover
     """Data class representing air conditioner status.
 
     Parameters
@@ -48,6 +45,7 @@ class JciHitachiAC(JciHitachiStatus):
         'sound_prompt': 30,
         'outdoor_temp': 33,
         'power_kwh': 40,
+        'freeze_clean': 57,
     }
 
     def __init__(self, status, default=-1):
@@ -360,8 +358,28 @@ class JciHitachiAC(JciHitachiStatus):
             return v
         return v / 10.0
 
+    @property
+    def freeze_clean(self):
+        """Freeze clean. Controlable.
 
-class JciHitachiDH(JciHitachiStatus):
+        Returns
+        -------
+        str
+            One of ("unsupported", "off", "on", "unknown").
+        """
+
+        v = self._status.get(self.idx['freeze_clean'], self._default)
+        if v == -1:
+            return "unsupported"
+        elif v == 0:
+            return "off"
+        elif v == 1:
+            return "on"
+        else:
+            return "unknown"
+
+
+class JciHitachiDH(JciHitachiStatus):  # pragma: no cover
     """Data class representing dehumidifier status.
 
     Parameters
@@ -790,7 +808,7 @@ class JciHitachiDH(JciHitachiStatus):
             return "unknown"
 
 
-class JciHitachiHE(JciHitachiStatus):
+class JciHitachiHE(JciHitachiStatus):  # pragma: no cover
     """Data class representing heat exchanger status. Not implemented.
 
     Parameters
@@ -807,7 +825,7 @@ class JciHitachiHE(JciHitachiStatus):
         super().__init__(status, default)
 
 
-class JciHitachiStatusSupport(JciHitachiStatus):
+class JciHitachiStatusSupport(JciHitachiStatus):  # pragma: no cover
     supported_type = {}
 
     def __init__(self, status, default=0):
@@ -882,7 +900,7 @@ class JciHitachiStatusSupport(JciHitachiStatus):
         return None
 
 
-class JciHitachiACSupport(JciHitachiStatusSupport):
+class JciHitachiACSupport(JciHitachiStatusSupport):  # pragma: no cover
     """Data model representing supported air conditioner status.
 
     Parameters
@@ -1163,7 +1181,7 @@ class JciHitachiACSupport(JciHitachiStatusSupport):
         return (v != 0, supports)
 
 
-class JciHitachiDHSupport(JciHitachiStatusSupport):
+class JciHitachiDHSupport(JciHitachiStatusSupport):  # pragma: no cover
     """Data model representing supported dehumidifier status.
 
     Parameters
@@ -1517,7 +1535,7 @@ class JciHitachiDHSupport(JciHitachiStatusSupport):
         return (v != 0, *supports)
 
 
-class JciHitachiHESupport(JciHitachiStatusSupport):
+class JciHitachiHESupport(JciHitachiStatusSupport):  # pragma: no cover
     """Data model representing supported heat exchanger status. Not implemented.
 
     Parameters
@@ -1534,59 +1552,389 @@ class JciHitachiHESupport(JciHitachiStatusSupport):
         super().__init__(status, default)
 
 
+STATUS_DICT = {
+    "AC": {
+        'DeviceType': {
+            "controlable": False,
+            "is_numeric": False,
+            "legacy_name": "DeviceType",
+            "id2str": {
+                1: "AC",
+                2: "DH",
+                3: "HE",
+                4: "PM25_PANEL",
+            }
+        },
+        'Switch': {
+            "controlable": True,
+            "is_numeric": False,
+            "legacy_name": "power",
+            "id2str": {
+                0: "off",
+                1: "on",
+            }
+        },
+        'Mode': {
+            "controlable": True,
+            "is_numeric": False,
+            "legacy_name": "mode",
+            "id2str": {
+                0: "cool",
+                1: "dry",
+                2: "fan",
+                3: "auto",
+                4: "heat",
+            }
+        },
+        'FanSpeed': {
+            "controlable": True,
+            "is_numeric": False,
+            "legacy_name": "air_speed",
+            "id2str": {
+                0: "auto",
+                1: "silent",
+                2: "low",
+                3: "moderate",
+                4: "high",
+            }
+        },
+        'TemperatureSetting': {
+            "controlable": True,
+            "is_numeric": True,
+            "legacy_name": "target_temp",
+        },
+        'IndoorTemperature': {
+            "controlable": False,
+            "is_numeric": True,
+            "legacy_name": "indoor_temp",
+        },
+        'SleepModeRemainingTime': {
+            "controlable": True,
+            "is_numeric": True,
+            "legacy_name": "sleep_timer",
+        },
+        'VerticalWindDirectionSwitch': {
+            "controlable": True,
+            "is_numeric": False,
+            "legacy_name": "vertical_wind_swingable",
+            "id2str": {
+                0: "disabled",
+                1: "enabled",
+            }
+        },
+        'VerticalWindDirectionSetting': {
+            "controlable": True,
+            "is_numeric": True,
+            "legacy_name": "vertical_wind_direction",
+        },
+        'HorizontalWindDirectionSetting': {
+            "controlable": True,
+            "is_numeric": False,
+            "legacy_name": 'horizontal_wind_direction',
+            "id2str": {
+                0: "auto",
+                1: "leftmost",
+                2: "middleleft",
+                3: "central",
+                4: "middleright",
+                5: "rightmost"
+            }
+        },
+        'MildewProof': {
+            "controlable": True,
+            "is_numeric": False,
+            "legacy_name": 'mold_prev',
+            "id2str": {
+                0: "disabled",
+                1: "enabled",
+            }
+        },
+        'QuickMode': {
+            "controlable": True,
+            "is_numeric": False,
+            "legacy_name": 'fast_op',
+            "id2str": {
+                0: "disabled",
+                1: "enabled",
+            }
+        },
+        'PowerSaving': {
+            "controlable": True,
+            "is_numeric": False,
+            "legacy_name":'energy_save',
+            "id2str": {
+                0: "disabled",
+                1: "enabled",
+            }
+        },
+        'ControlTone': {
+            "controlable": True,
+            "is_numeric": False,
+            "legacy_name":'sound_prompt',
+            "id2str": {
+                0: "enabled",
+                1: "disabled",
+            }
+        },
+        'PowerConsumption': {
+            "controlable": False,
+            "is_numeric": True,
+            "legacy_name":'power_kwh',
+        },
+        'TaiseiaError': {
+            "controlable": False,
+            "is_numeric": True,
+            "legacy_name": None,
+        },
+        'FilterElapsedHour': {
+            "controlable": False,
+            "is_numeric": True,
+            "legacy_name": None,
+        },
+        'CleanSwitch': {
+            "controlable": True,
+            "is_numeric": False,
+            "legacy_name":'freeze_clean',
+            "id2str": {
+                0: "off",
+                1: "on",
+            }
+        },
+        'CleanNotification': {
+            "controlable": False,
+            "is_numeric": True,
+            "legacy_name": None,
+        },
+        'CleanStatus': {
+            "controlable": False,
+            "is_numeric": True,
+            "legacy_name": None,
+        },
+        'Error': {
+            "controlable": False,
+            "is_numeric": True,
+            "legacy_name": None,
+        },
+        'max_temp': {
+            "controlable": False,
+            "is_numeric": True,
+            "legacy_name": 'max_temp',
+        },
+        'min_temp': {
+            "controlable": False,
+            "is_numeric": True,
+            "legacy_name": 'min_temp',
+        },
+    },
+    "DH": {
+        'DeviceType': {
+            "controlable": False,
+            "is_numeric": False,
+            "legacy_name": "DeviceType",
+            "id2str": {
+                1: "AC",
+                2: "DH",
+                3: "HE",
+                4: "PM25_PANEL",
+            }
+        },
+        'Switch': {
+            "controlable": True,
+            "is_numeric": False,
+            "legacy_name": "power",
+            "id2str": {
+                0: "off",
+                1: "on",
+            }
+        },
+        'Mode': {
+            "controlable": True,
+            "is_numeric": False,
+            "legacy_name": "mode",
+            "id2str": {
+                0: "auto",
+                1: "custom",
+                2: "continuous",
+                3: "clothes_dry",
+                4: "air_purify",
+                5: "mold_prev",
+                8: "low_humidity",
+                9: "eco_comfort",
+            }
+        },
+        'FanSpeed': {
+            "controlable": True,
+            "is_numeric": False,
+            "legacy_name": "air_speed",
+            "id2str": {
+                0: "auto",
+                1: "silent",
+                2: "low",
+                3: "moderate",
+                4: "high",
+            }
+        },
+        'MildewProof': {
+            "controlable": True,
+            "is_numeric": False,
+            "legacy_name": "mold_prev",
+            "id2str": {
+                0: "disabled",
+                1: "enabled",
+            }
+        },
+        'ControlTone': {
+            "controlable": True,
+            "is_numeric": False,
+            "legacy_name": 'sound_control',
+            "id2str": {
+                0: "silent",
+                1: "button",
+                2: "button+waterfull",
+            }
+        },
+        'SaaControlTone': {  # currently not supported
+            "controlable": False,
+            "is_numeric": False,
+            "legacy_name": None,
+            "id2str": {
+            }
+        },
+        'PowerConsumption': {
+            "controlable": False,
+            "is_numeric": True,
+            "legacy_name": 'power_kwh',
+        },
+        'Ion': {  # currently not supported
+            "controlable": False,
+            "is_numeric": False,
+            "legacy_name": None,
+            "id2str": {
+            }
+        },
+        'HumiditySetting': {
+            "controlable": True,
+            "is_numeric": True,
+            "legacy_name": 'target_humidity',
+        },
+        'AutoWindDirection': {
+            "controlable": True,
+            "is_numeric": False,
+            "legacy_name": 'wind_swingable',
+            "id2str": {
+                0: "disabled",
+                1: "enabled",
+            }
+        },
+        'KeypadLock': {
+            "controlable": True,
+            "is_numeric": False,
+            "legacy_name": None,
+            "id2str": {
+            }
+        },
+        'DisplayBrightness': {
+            "controlable": True,
+            "is_numeric": False,
+            "legacy_name": 'display_brightness',
+            "id2str": {
+                0: "bright",
+                1: "dark",
+                2: "off",
+                3: "all_off",
+            }
+        },
+        'FilterControl': {
+            "controlable": True,
+            "is_numeric": False,
+            "legacy_name": 'air_cleaning_filter',
+            "id2str": {
+                0: "disabled",
+                1: "enabled",
+            }
+        },
+        'PM25': {
+            "controlable": False,
+            "is_numeric": True,
+            "legacy_name": 'pm25_value',
+        },
+        'IndoorHumidity': {
+            "controlable": False,
+            "is_numeric": True,
+            "legacy_name": 'indoor_humidity',
+        },
+        'SideAirOutlet': {
+            "controlable": False,
+            "is_numeric": False,
+            "legacy_name": 'side_vent',
+            "id2str": {
+                0: "off",
+                1: "on",
+            }
+        },
+        'Defrost': {
+            "controlable": True,
+            "is_numeric": False,
+            "legacy_name": None,
+            "id2str": {
+            }
+        },
+        'SmellIndex': {
+            "controlable": False,
+            "is_numeric": False,
+            "legacy_name": 'odor_level',
+            "id2str": {
+                0: "low",
+                1: "middle",
+                2: "high",
+            }
+        },
+        'CleanFilterNotification': {
+            "controlable": True,
+            "is_numeric": False,
+            "legacy_name": 'clean_filter_notify',
+            "id2str": {
+                0: "disabled",
+                1: "enabled",
+            }
+        },
+        'TankFullNotification': {
+            "controlable": False,
+            "is_numeric": False,
+            "legacy_name": 'water_full_warning',
+            "id2str": {
+                0: "off",  # not activated
+                1: "on",  # activated
+            }
+        },
+        'TaiseiaError': {
+            "controlable": False,
+            "is_numeric": True,
+            "legacy_name": None,
+        },
+        'Error': {
+            "controlable": False,
+            "is_numeric": True,
+            "legacy_name": 'error_code',
+        },
+        'max_humidity': {
+            "controlable": False,
+            "is_numeric": True,
+            "legacy_name": 'max_humidity',
+        },
+        'min_humidity': {
+            "controlable": False,
+            "is_numeric": True,
+            "legacy_name": 'min_humidity',
+        },
+    },
+    "HE": {
+
+    }
+}
+PREPARED_STATUS_DICT = None
 
 class JciHitachiAWSStatus:
-    compatibility_mapping = {
-        "AC": {
-            'DeviceType': None,
-            'Switch': 'power',
-            'Mode': 'mode',
-            'FanSpeed': 'air_speed',
-            'TemperatureSetting': 'target_temp',
-            'IndoorTemperature': 'indoor_temp',
-            'OutdoorTemperature': 'outdoor_temp',
-            'SleepModeRemainingTime': 'sleep_timer',
-            'VerticalWindDirectionSwitch': 'vertical_wind_swingable', 
-            'VerticalWindDirectionSetting': 'vertical_wind_direction',
-            'HorizontalWindDirectionSetting': 'horizontal_wind_direction',
-            'MildewProof': 'mold_prev',
-            'QuickMode': 'fast_op',
-            'PowerSaving': 'energy_save',
-            'ControlTone': 'sound_prompt',
-            'PowerConsumption': 'power_kwh',
-            'TaiseiaError': None,
-            'FilterElapsedHour': None,
-            'CleanSwitch': None,
-            'CleanNotification': None,
-            'CleanStatus': None,
-            'Error': None,
-        },
-        "DH": {
-            'DeviceType': None,
-            'Switch': 'power',
-            'Mode': 'mode',
-            'FanSpeed': 'air_speed',
-            'MildewProof': 'mold_prev',
-            'ControlTone': 'sound_control',
-            'SaaControlTone': None,
-            'PowerConsumption': 'power_kwh',
-            'Ion': None,
-            'HumiditySetting': 'target_humidity',
-            'AutoWindDirection': 'wind_swingable',
-            'KeypadLock': None,
-            'DisplayBrightness': 'display_brightness',
-            'FilterControl': 'air_cleaning_filter',
-            'PM25': 'pm25_value',
-            'IndoorHumidity': 'indoor_humidity',
-            'SideAirOutlet': 'side_vent',
-            'Defrost': None,
-            'SmellIndex': 'odor_level',
-            'CleanFilterNotification': 'clean_filter_notify',
-            'TankFullNotification': 'water_full_warning',
-            'TaiseiaError': None,
-            'Error': 'error_code',
-        }
-    }
     device_type_mapping = {
         1: "AC",
         2: "DH",
@@ -1594,23 +1942,40 @@ class JciHitachiAWSStatus:
         4: "PM25_PANEL",
     }
 
-    def __init__(self, status: dict) -> None:
-        self._status: dict = self._preprocess(status)
+    def __init__(self, raw_status: dict, legacy=False) -> None:
+        self._status: dict = raw_status if legacy else self._preprocess(raw_status)
+        self._device_type: str = self._status["DeviceType"]
 
     def __getattr__(self, name):
-        return self._status[name]
+        return self._status.get(name, "unsupported")
     
     def __repr__(self) -> str:
         return str(self._status)
 
-    def _preprocess(self, status):
+    def _preprocess(self, raw_status):
         # device type
-        if status.get("DeviceType"):
-            status["DeviceType"] = self.device_type_mapping[status["DeviceType"]]
+        if "DeviceType" not in raw_status or \
+            ("DeviceType" in raw_status and raw_status["DeviceType"] not in self.device_type_mapping):
+            raise AttributeError("`DeviceType` isn't in the raw status or has an invalid value.")
         
-        if not status.get("OutdoorTemperature"):
-            status["OutdoorTemperature"] = 0
+        if "PowerConsumption" in raw_status:
+            raw_status["PowerConsumption"] /= 10.0
+
+        status = {}
+        device_type = self.device_type_mapping[raw_status["DeviceType"]]
+        for key, value in raw_status.items():
+            if key in STATUS_DICT[device_type]:
+                if STATUS_DICT[device_type][key]["is_numeric"]:
+                    status[key] = value
+                else:
+                    status[key] = STATUS_DICT[device_type][key]["id2str"].get(value, "unknown")
         
+        if device_type == "AC":
+            status["max_temp"] = 32
+            status["min_temp"] = 16
+        elif device_type == "DH":
+            status["max_humidity"] = 70
+            status["min_humidity"] = 40
         return status
 
     @property
@@ -1626,41 +1991,54 @@ class JciHitachiAWSStatus:
         return self._status
     
     @property
-    def legacy_status_class(self):
-        """All legacy status used by the old API.
+    def legacy_status(self):
+        """All legacy status name used by the old API.
 
         Returns
         -------
-        JciHitachiStatus
-            status.
+        JciHitachiAWSStatus
+            Status with legacy name.
         """
 
-        class mock_idx(dict):
-            def __getitem__(self, key):
-                return key
-
-        mapped_status = {self.compatibility_mapping[self._status["DeviceType"]].get(key): value for key, value in self._status.items()}
-
-        if self._status["DeviceType"] == "AC":
-            if None in mapped_status:
-                del mapped_status[None]
-            ac = JciHitachiAC(mapped_status)
-            ac.idx = mock_idx(ac.idx)
-            return ac
-        elif self._status["DeviceType"] == "DH":
-            if None in mapped_status:
-                del mapped_status[None]
-            dh = JciHitachiDH(mapped_status)
-            dh.idx = mock_idx(dh.idx)
-            return dh
-        return None
+        status = {}
+        device_type = self._status["DeviceType"]
+        for status_name, status_value in self._status.items():
+            if status_name in STATUS_DICT[device_type] and \
+                STATUS_DICT[device_type][status_name]["legacy_name"] is not None:
+                    status.update({STATUS_DICT[device_type][status_name]["legacy_name"]: status_value})
+        return JciHitachiAWSStatus(status, legacy=True)
 
     @staticmethod
-    def convert_old_to_new(device_type, old_status_name):
-        for key, value in __class__.compatibility_mapping[device_type].items():
-            if value == old_status_name:
-                return key
-        return None
+    def str2id(device_type: str, status_name: str, status_value: int = None, status_str_value: str = None):
+        is_valid = (status_value is not None) ^ (status_str_value is not None)
+
+        # Name check
+        if status_name not in STATUS_DICT[device_type]:
+            legacy2new_dict = {specs["legacy_name"]: new_status_name for new_status_name, specs in STATUS_DICT[device_type].items()}
+            if status_name in legacy2new_dict:
+                status_name = legacy2new_dict[status_name]
+            else:
+                is_valid = False
+
+        # Value check
+        if is_valid:
+            if status_str_value is not None:
+                str2id_dict = {value:key for key, value in STATUS_DICT[device_type][status_name]["id2str"].items()}
+                if status_str_value in str2id_dict:
+                    status_value = str2id_dict[status_str_value]
+                else:
+                    is_valid = False
+            else:
+                if not STATUS_DICT[device_type][status_name]["is_numeric"] and status_value not in STATUS_DICT[device_type][status_name]["id2str"]:
+                    is_valid = False
+
+        return is_valid, status_name, status_value
+
+    def set_new_status(self, name: str, value: int):
+        if STATUS_DICT[self._device_type][name]["is_numeric"]:
+            self._status[name] = value
+        else:
+            self._status[name] = STATUS_DICT[self._device_type][name]["id2str"].get(value, "unknown")
 
 class JciHitachiAWSStatusSupport:
     extended_mapping = {
@@ -1670,14 +2048,13 @@ class JciHitachiAWSStatusSupport:
         "FindMe": None,
     }
 
-    compatibility_mapping = copy.deepcopy(JciHitachiAWSStatus.compatibility_mapping).update(extended_mapping)
     device_type_mapping = JciHitachiAWSStatus.device_type_mapping
 
     def __init__(self, status: dict) -> None:
         self._status: dict = self._preprocess(status)
 
     def __getattr__(self, name):
-        return self._status[name]
+        return self._status.get(name, "unsupported")
     
     def __repr__(self) -> str:
         return str(self._status)
@@ -1687,9 +2064,6 @@ class JciHitachiAWSStatusSupport:
         if status.get("DeviceType"):
             status["DeviceType"] = self.device_type_mapping[status["DeviceType"]]
         
-        if not status.get("OutdoorTemperature"):
-            status["OutdoorTemperature"] = 0
-        
         status["Brand"] = "HITACHI"
         
         return status
